unit ChWallpaperU;

interface

uses
  ChWpCommon, About,
  ShlObj, ActiveX, ComObj,
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, FileCtrl, ExtCtrls, Buttons;

const
  AutoChangeShortcutName = 'Wallpaper Manager AUTO mode.lnk';

type
  TChWallPaperF = class(TForm)
    Panel1: TPanel;
    DriveComboBox1: TDriveComboBox;
    WPaperDLB: TDirectoryListBox;
    WPaperFLB: TFileListBox;
    Panel2: TPanel;
    Panel3: TPanel;
    Panel4: TPanel;
    BitBtn1: TBitBtn;
    WpI: TImage;
    ChWpMethodRG: TRadioGroup;
    Panel5: TPanel;
    SResolL: TLabel;
    AboutBB: TBitBtn;
    GroupBox1: TGroupBox;
    WPDirL: TLabel;
    WPCurWPL: TLabel;
    Panel6: TPanel;
    SetWallPBB: TBitBtn;
    SetWpL: TLabel;
    Panel7: TPanel;
    SetWPDirBB: TBitBtn;
    SetWpDirL: TLabel;
    PicResL: TLabel;
    AutoChangeCB: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure SetWPDirBBClick(Sender: TObject);
    procedure WPaperFLBChange(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Panel2Resize(Sender: TObject);
    procedure SetWallPBBClick(Sender: TObject);
    procedure AboutBBClick(Sender: TObject);
    procedure ChWpMethodRGClick(Sender: TObject);
    procedure WPaperDLBChange(Sender: TObject);
    procedure AutoChangeCBClick(Sender: TObject);
  private
    FProgDir : string;
    FWallpaperDir: string;
    FWallpaper: string;
    FPicture: TPicture;
    FEnableAutochangeConfirm : boolean;
    FEnableAutochange: boolean;
    procedure SetWallpaper(const Value: string);
    procedure SetWallpaperDir(const Value: string);
    procedure SetPicture(const Value: TPicture);
    procedure DrawPicture;
    procedure ShowOptionInfo;
    procedure SetEnableAutochange(const Value: boolean);
    function GetEnableAutochange : boolean;
  public
    property Wallpaper : string read FWallpaper write SetWallpaper;
    property WallpaperDir : string read FWallpaperDir write SetWallpaperDir;
    property Picture : TPicture read FPicture write SetPicture;
    property EnableAutochange : boolean read GetEnableAutochange write SetEnableAutochange;
  end;

var
  ChWallPaperF: TChWallPaperF;

implementation

{$R *.DFM}

procedure TChWallPaperF.FormCreate(Sender: TObject);
var
  CurWP : string; CWI : integer;
begin
  FWallpaperDir := '';
  FWallpaper := '';

  FEnableAutochangeConfirm := false;
  AutoChangeCB.Checked := EnableAutochange;
  FEnableAutochangeConfirm := true;

  SResolL.Caption := Format( 'Screen: %dx%d',
                             [Screen.Width, Screen.Height]);

  FPicture := TPicture.Create;

  ChWpMethodRG.ItemIndex := ord( WpOptions.ChangeMethod);

  WallpaperDir := WpOptions.WallpapersDir;
  CurWP := ExtractFileName( WpOptions.CurrentWalpaper);
  CWI := WPaperFLB.Items.IndexOf( CurWP);
  if CWI >= 0 then WPaperFLB.ItemIndex := CWI;
  WPaperDLBChange( Self);
  WPaperFLBChange( Self);
  ShowOptionInfo;
end;

procedure TChWallPaperF.SetPicture(const Value: TPicture);
begin
  FPicture := Value;
end;

procedure TChWallPaperF.SetWallpaper(const Value: string);
begin
  if FWallpaper = Value then exit;
  FWallpaper := Value;
  Picture.LoadFromFile( FWallpaper);
  DrawPicture;
end;

procedure TChWallPaperF.SetWallpaperDir(const Value: string);
begin
  if FWallpaperDir = Value then exit;
  FWallpaperDir := Value;
  WPaperDLB.Directory := FWallpaperDir;
end;

procedure TChWallPaperF.SetWPDirBBClick(Sender: TObject);
begin
  WpOptions.WallpapersDir := WPaperDLB.Directory;
  WriteOptions;
  WallpaperDir := WPaperDLB.Directory;
  ShowOptionInfo;
end;

procedure TChWallPaperF.WPaperFLBChange(Sender: TObject);
begin
  SetWpL.Caption := ExtractFileName( WPaperFLB.FileName);
  PicResL.Caption := '';
  if WPaperFLB.FileName = '' then exit;
  Wallpaper := WPaperFLB.FileName;
  PicResL.Caption := Format( 'Picture: %dx%d', [Picture.Width, Picture.Height]);
end;

procedure TChWallPaperF.FormDestroy(Sender: TObject);
begin
  FPicture.Free;
end;

function CalcPicSize( PicW, PicH, W, H : integer) : TPoint;
var
  PicAspect : double;
  ByWidth, ByHeight : TPoint;
begin
  if ( W = 0) AND ( H = 0) then begin
    Result.X := PicW;
    Result.Y := PicH;
    exit;
    end;
  PicAspect := PicW / PicH;

  ByWidth.X := W; ByWidth.Y := round( W / PicAspect);
  ByHeight.Y := H; ByHeight.X := round( H * PicAspect);

  if ByWidth.Y <= H then Result := ByWidth
  else                   Result := ByHeight;
end;

procedure TChWallPaperF.DrawPicture;
var PicDim : TPoint; TmpBmp : TBitMap;
begin
  if Wallpaper = '' then exit;
  PicDim := CalcPicSize( Picture.Width, Picture.Height,
                         WpI.Width - 10, WpI.Height - 10);
  TmpBmp := TBitMap.Create;
  try
    TmpBmp.Width := PicDim.X;
    TmpBmp.Height := PicDim.Y;
    TmpBmp.Canvas.StretchDraw( TmpBmp.Canvas.ClipRect, Picture.Graphic);
    WpI.Picture.Graphic := TmpBmp;
  finally TmpBmp.Free; end;
end;

procedure TChWallPaperF.Panel2Resize(Sender: TObject);
begin
  DrawPicture;
end;

procedure TChWallPaperF.SetWallPBBClick(Sender: TObject);
begin
  if Wallpaper = '' then exit;
  ChangeWallpaper( Wallpaper);
  WpOptions.CurrentWalpaper := Wallpaper;
  WriteOptions;
  ShowOptionInfo;
end;

procedure TChWallPaperF.AboutBBClick(Sender: TObject);
begin
  AboutBox := TAboutBox.Create( Self);
  AboutBox.ShowModal;
  AboutBox.Free;
end;

procedure TChWallPaperF.ChWpMethodRGClick(Sender: TObject);
begin
  WpOptions.ChangeMethod := TWChangeMethod( ChWpMethodRG.ItemIndex);
  WriteOptions;
end;

procedure TChWallPaperF.ShowOptionInfo;
begin
  WPDirL.Caption := Format( 'Walpaper directory: %s', [WpOptions.WallpapersDir]);
  WPCurWPL.Caption := Format( 'Current wallpaper: %s', [WpOptions.CurrentWalpaper]);
end;

procedure TChWallPaperF.WPaperDLBChange(Sender: TObject);
begin
  SetWpDirL.Caption := WPaperDLB.Directory;
end;

procedure TChWallPaperF.AutoChangeCBClick(Sender: TObject);
var S : string;
begin
  if NOT FEnableAutochangeConfirm then exit;
  if AutoChangeCB.Checked then S := 'Enable'
  else                         S := 'Disable';
  S := S + ' autochange wallpaper On StartUp?';
  if Application.MessageBox( PChar( S), 'Confirm', MB_OKCANCEL) <> IDOK then begin
    FEnableAutochangeConfirm := false;
    AutoChangeCB.Checked := NOT AutoChangeCB.Checked;
    FEnableAutochangeConfirm := true;
    exit;
    end;
  EnableAutochange := AutoChangeCB.Checked; 
end;

procedure TChWallPaperF.SetEnableAutochange(const Value: boolean);
var
  StartupDir : string;
  MyObject  : IUnknown;
  MySLink   : IShellLink;
  MyPFile   : IPersistFile;
  FileName  : String;
  WFileName : WideString;
begin
  StartupDir := ShellFolderDir( 'Startup');
  if Value then begin { create shortcut in \Start\Programs\StartUp folder}
    MyObject := CreateComObject(CLSID_ShellLink);
    MySLink := MyObject as IShellLink;
    MyPFile := MyObject as IPersistFile;
    FileName := ParamStr( 0);
    with MySLink do begin
      SetArguments( smcAuto);
      SetPath(PChar(FileName));
      SetWorkingDirectory(PChar(ExtractFilePath(FileName)));
      end;
    WFileName := StartupDir + '\' + AutoChangeShortcutName;
    MyPFile.Save(PWChar(WFileName),False);
    end
  else begin { Remove shortcut from \Start\Programs\StartUp folder}
    if GetEnableAutochange then DeleteFile( StartupDir + '\' + AutoChangeShortcutName);
    end;
end;

function TChWallPaperF.GetEnableAutochange: boolean;
begin
  Result := FileExists( ShellFolderDir( 'Startup') + '\' + AutoChangeShortcutName);
end;

end.
